/*------------------------------------------------------------------
    (c) 2000 Innov8 Computer Software. and
    (c) 2000 Lichtenberg Consulting
    All rights reserved.  Permission granted to freely use and
    distribute so long as this notice appears in tact.

    File Name:     DataPump.p
    Author:        Geoff Crawford and Steve Licthenberg
                   with design influence from Brian Laferte
    Creation Date: 8/17/00

    Called From:
    Programs Called:

    Inputs Parameters:  DataTable - a handle to a buffer
    Outputs Parameters: No parameters, but a visualization
                        of the table results.


    Program Purpose:    Given a handle to a generic buffer,
                        run through all of the records
                        visualizing each record.

    Description Of Design: This is a single program implementation
                           of the DataPump.  Normally three sets
                           of Procedures are used and are made to
                           be instantiated as a set of SUPER's:

                           TablePump.p - Pre/Post/VisualizeTable
                           RecordPump.p - Pre/Post/VisualizeRecord
                           FieldPump.p - Pre/Post/VisualizeField

                           A SUPER version would then call each
                           of those persistently with customized
                           versions of each of the three for the
                           various types of visualizations.  Use
                           logic in that master program to piece
                           the set of SUPER's you want for UI
                           independt visualization. This particular
                           program can still be run persistant,
                           but does not need to be SUPER.

                           The idea is simple - visualize the
                           table by visualizing each record.
                           Use a dynamic query established at the
                           top level to run through each record.
                           Pre and Post Procedures are available
                           to do processing before and after the
                           traversal of the records.  Something
                           like an HTML table would need this for
                           the <TABLE> tags.

                           Each record is visualized by using
                           the buffer handle to run through each
                           field in the record.  Calls are made then
                           to visualize each field.  Pre/Post
                           Procedures available for conditional
                           validation, translation, etc.

    Modification History: 8/17/00 Merged original dynamic query code
                                  from Steve with atomic low level
                                  procedures from Geoff.
  -------------------------------------------------------------------*/

subscribe to "Getdatapumphandle":U anywhere.

PROCEDURE getdatapushhandle :
 DEF OUTPUT PARAMETER op_hMySuperProcedure AS HANDLE NO-UNDO.
 ASSIGN op_hMySuperProcedure = THIS-PROCEDURE:HANDLE.
END PROCEDURE.


PROCEDURE PreVisualizeTable.
    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "<table border=~"1~">".
END. /* PreVisualizeTable */


PROCEDURE VisualizeTable.


    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    DEFINE VARIABLE DataTableQuery AS HANDLE NO-UNDO.
    DEFINE VARIABLE isok AS LOGICAL NO-UNDO.

    /* Create a dynamic query using the handle passed in. */
    run PreVisualizeTable in target-procedure (input DataTable,
        input filenm,
        input keyfld).
    CREATE QUERY DataTableQuery.

    isok = DataTableQuery:SET-BUFFERS(DataTable). /* assign the buffer to the query */
    isok = DataTableQuery:QUERY-PREPARE("for each " + DataTable:NAME + " no-lock"). /* query the entire table */
    isok = DataTableQuery:QUERY-OPEN.  /* open the query */

    /* Run through each record, calling down one level to visualize each record */

    REPEAT:

        /* Get the next record.  Leave the Procedure if we're done. */

        isok = DataTableQuery:GET-NEXT.
        IF NOT isok THEN RETURN.
        IF DataTableQuery:QUERY-OFF-END THEN RETURN.

        /* Visualize the Record */

        RUN VisualizeRecord in target-procedure (INPUT DataTable,
        input filenm,
        input keyfld).

    END.
    run PostVisualizeTable in target-procedure (input DataTable,
        input filenm,
        input keyfld).
END. /* Visualize Table */



PROCEDURE PostVisualizeTable.


    DEFINE INPUT PARAMETER DataTable AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "</table>".
END. /* PostVisualizeTable */


PROCEDURE PreVisualizeRecord.
    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "<tr>"
END. /* PreVisualizeRecord */



PROCEDURE VisualizeRecord.


    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    DEFINE VARIABLE DataField AS HANDLE NO-UNDO.
    DEFINE VARIABLE i AS INTEGER NO-UNDO.

    RUN PreVisualizeRecord in target-procedure (INPUT RecordHandle,
        input filenm,
        input keyfld).

    /* Use the Buffer-Handle to dynamically grab each one of the
       data field handles. Perhaps this could be done more
       efficiently by storing the set of record handles only
       once since the Handles do not change.  However, the
       following is the clearest code. */

    /* This code assumes you've created a temp-table just the
       way you want it, without any extraneous fields.  Create
       a different version of the code below if that's not the case. */

    DO i = 1 TO RecordHandle:NUM-FIELDS:

        DataField = RecordHandle:BUFFER-FIELD(i). /* get the handle by the field number, not name */

        /* Visualize the Field */

        RUN VisualizeField in target-procedure (INPUT DataField,
        input filenm,
        input keyfld).

    END.

    RUN PostVisualizeRecord in target-procedure (INPUT RecordHandle,
        input filenm,
        input keyfld).

END. /* Visualize Record */



PROCEDURE PostVisualizeRecord.


    DEFINE INPUT PARAMETER RecordHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "</tr>".
END. /* PostVisualizeRecord */



PROCEDURE PreVisualizeField.


    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "<td>".
END. /* PreVisualizeField */



PROCEDURE VisualizeField.


    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    RUN PreVisualizeField in target-procedure (INPUT FieldHandle,
        input filenm,
        input keyfld).

    /* Actual Visualization here is a simple Display.
       This isn't really all that useful, but it's
       mainly a demo version of VisualizeField. */

    if not valid-handle(fieldhandle) then {&out} "nbsp;".
    else {&out} FieldHandle:BUFFER-VALUE.

    RUN PostVisualizeField in target-procedure (INPUT FieldHandle,
        input filenm,
        input keyfld).

END. /* Visualize Field */



PROCEDURE PostVisualizeField.


    DEFINE INPUT PARAMETER FieldHandle AS HANDLE NO-UNDO.
    def input parameter       filenm        as char  no-undo.
    def input parameter       keyfld        as char  no-undo.

    {&out} "</td>".
END. /* PostVisualizeField */


